/***************************************************************************
 *   Copyright (C) 2008, 2009, 2010 by Miguel Escudero Gestal
 *   miguelescudero@users.sourceforge.net
 *
 *   This file is part of tubularix
 *
 *   tubularix is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   tubularix is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "gametypedialog.h"
#include "gameoptionsdialog.h"
#include <QDateTime>
#include <QColorDialog>
#include <QGridLayout>
#include <QLocale>
#include <QSettings>
#include <QTime>
#include <QTimer>
//#include <QLatin1String>

#include <iostream>
using namespace std;


//public


MainWindow::MainWindow(QWidget *parent)
	: QMainWindow(parent), ui(new Ui::MainWindowClass)
{
	ui->setupUi(this);
	qsrand(QTime::currentTime().msec());
	currentGameType = 1;
	currentGameMode = 1;
	startLevel = 1;
	for(int i = 0; i < 9; i++)
		colors[i] = QColor(0, 0, 0);

	gameBoard = new GameBoard(this);
	gameBoard2 = new GameBoard(this);
	//gameBoard->setMinimumWidth(width() / 8);
	//gameBoard2->setMinimumWidth(width() / 8);
	splitter = new QSplitter(Qt::Horizontal, this);
	//splitter->setAutoFillBackground(false);
	//splitter->setBackgroundRole(QPalette::NoRole);
	//splitter->setForegroundRole(QPalette::NoRole);
	splitter->setFrameShape(QFrame::StyledPanel);
	splitter->setFrameShadow(QFrame::Sunken);
	//splitter->setOpaqueResize(false);
	//splitter->setChildrenCollapsible(false);
	splitter->setFocusPolicy(Qt::NoFocus);
	splitter->addWidget(gameBoard2);
	splitter->addWidget(gameBoard);
	QList <int> list1;
	list1<<10<<10;
	splitter->setSizes(list1);
	setCentralWidget(splitter);

	connect(ui->actionQuit, SIGNAL(triggered()), this, SLOT(close()));
	connect(ui->actionShowHighScores, SIGNAL(triggered()), this, SLOT(showHighScores()));
	connect(ui->actionSelectLanguage, SIGNAL(triggered()), this, SLOT(selectLanguage()));
	connect(ui->actionRestoreSizeSettings, SIGNAL(triggered()), this, SLOT(restoreSizeSettings()));
	connect(ui->actionAbout, SIGNAL(triggered()), this, SLOT(about()));
	connect(ui->actionAboutQt, SIGNAL(triggered()), this, SLOT(aboutQt()));
	connect(ui->actionNewGame, SIGNAL(triggered()), this, SLOT(startGame()));
	connect(ui->actionSelectGameType, SIGNAL(triggered()), this, SLOT(selectGameType()));
	connect(ui->actionGameOptions, SIGNAL(triggered()), this, SLOT(showGameOptions()));
	connect(ui->actionFullScreen, SIGNAL(triggered()), this, SLOT(changeFullScreen()));

	//Game board player 1
	connect(ui->actionActivateAntialiasing, SIGNAL(toggled(bool)), gameBoard, SLOT(setAntialiasing(bool)));
	connect(this, SIGNAL(gameTypeChanged(int)), gameBoard, SLOT(setGameType(int)));
	connect(this, SIGNAL(gameModeChanged(int)), gameBoard, SLOT(setGameMode(int)));
	connect(this, SIGNAL(startLevelChanged(int)), gameBoard, SLOT(setStartLevel(int)));
	connect(ui->actionShowMap, SIGNAL(toggled(bool)), gameBoard, SLOT(setShowMap(bool)));
	connect(ui->actionPiecesAppearingGradually, SIGNAL(toggled(bool)),
			gameBoard, SLOT(setPiecesAppearGradually(bool)));
	connect(ui->actionShowGhostPiece, SIGNAL(toggled(bool)), gameBoard, SLOT(setShowGhostPiece(bool)));
	connect(this, SIGNAL(rotationStyleChanged(int)), gameBoard, SLOT(setRotationStyle(int)));
	connect(this, SIGNAL(colorsChanged(QColor*)), gameBoard, SLOT(setColors(QColor*)));

	connect(gameBoard, SIGNAL(fullScreenChanged()), this, SLOT(changeFullScreen()));
	connect(gameBoard, SIGNAL(mouseMovedTop()), this, SLOT(showExtraElements()));
	connect(gameBoard, SIGNAL(mouseMovedBottom()), this, SLOT(hideExtraElements()));
	connect(gameBoard, SIGNAL(paintingNeeded()), this, SLOT(update()));

	connect(gameBoard, SIGNAL(gameStopped()), this, SLOT(endGame()));
	connect(gameBoard, SIGNAL(nextPieceNeeded()), this, SLOT(addPiece()));

	//Game board player 2
	connect(ui->actionActivateAntialiasing, SIGNAL(toggled(bool)), gameBoard2, SLOT(setAntialiasing(bool)));
	connect(this, SIGNAL(gameTypeChanged(int)), gameBoard2, SLOT(setGameType(int)));
	connect(this, SIGNAL(gameModeChanged(int)), gameBoard2, SLOT(setGameMode(int)));
	connect(this, SIGNAL(startLevelChanged(int)), gameBoard2, SLOT(setStartLevel(int)));
	connect(ui->actionShowMap, SIGNAL(toggled(bool)), gameBoard2, SLOT(setShowMap(bool)));
	connect(ui->actionPiecesAppearingGradually, SIGNAL(toggled(bool)),
			gameBoard2, SLOT(setPiecesAppearGradually(bool)));
	connect(ui->actionShowGhostPiece, SIGNAL(toggled(bool)), gameBoard2, SLOT(setShowGhostPiece(bool)));
	connect(this, SIGNAL(rotationStyleChanged(int)), gameBoard2, SLOT(setRotationStyle(int)));
	connect(this, SIGNAL(colorsChanged(QColor*)), gameBoard2, SLOT(setColors(QColor*)));

	connect(gameBoard2, SIGNAL(fullScreenChanged()), this, SLOT(changeFullScreen()));
	connect(gameBoard2, SIGNAL(mouseMovedTop()), this, SLOT(showExtraElements()));
	connect(gameBoard2, SIGNAL(mouseMovedBottom()), this, SLOT(hideExtraElements()));
	connect(gameBoard2, SIGNAL(paintingNeeded()), this, SLOT(update()));

	connect(gameBoard2, SIGNAL(gameStopped()), this, SLOT(endGame()));
	connect(gameBoard2, SIGNAL(nextPieceNeeded()), this, SLOT(addPiece()));

	/*
	//network
	nextClientBlockSize = 0;
	nextServerBlockSize = 0;
	connect(&tcpServer, SIGNAL(newConnection()), this, SLOT(acceptConnection()));
	connect(&tcpClient, SIGNAL(readyRead()), this, SLOT(processClientPendingStreams()));
	connect(&tcpClient, SIGNAL(disconnected()), this, SLOT(closeConnection()));
	*/

	readSettings();

	//key controllers
	keyFirstTimerPlayer1 = new QTimer(this);
	keyFirstTimerPlayer2 = new QTimer(this);
	connect(keyFirstTimerPlayer1, SIGNAL(timeout()), this, SLOT(setRepeatKeyPlayer1()));
	connect(keyFirstTimerPlayer2, SIGNAL(timeout()), this, SLOT(setRepeatKeyPlayer2()));
	keyFirstTimerPlayer1->setInterval(keyRepetitionDelay);
	keyFirstTimerPlayer2->setInterval(keyRepetitionDelay);
	keyRepeatTimerPlayer1 = new QTimer(this);
	keyRepeatTimerPlayer2 = new QTimer(this);
	connect(keyRepeatTimerPlayer1, SIGNAL(timeout()), this, SLOT(updateRepeatKeyPressPlayer1()));
	connect(keyRepeatTimerPlayer2, SIGNAL(timeout()), this, SLOT(updateRepeatKeyPressPlayer2()));
	keyRepeatTimerPlayer1->setInterval(1000 / keyRepetitionRate);
	keyRepeatTimerPlayer2->setInterval(1000 / keyRepetitionRate);

	//ai movement
	aiMovementTimer1 = new QTimer(this);
	connect(aiMovementTimer1, SIGNAL(timeout()), this, SLOT(aiMovement1()));
	aiMovementTimer1->setInterval(1000 / 10);
	aiMovementTimer2 = new QTimer(this);
	connect(aiMovementTimer2, SIGNAL(timeout()), this, SLOT(aiMovement2()));
	aiMovementTimer2->setInterval(1000 / 10);

	connect(ui->actionActivateAiPlayer1, SIGNAL(toggled(bool)), this, SLOT(setAiPlayer1(bool)));
	connect(ui->actionActivateAiPlayer2, SIGNAL(toggled(bool)), this, SLOT(setAiPlayer2(bool)));

	//setFocus();
	//if(currentGameType <= 2)
		startGame();

	/*
	if(currentGameType == 3)
	{
		if(gameBoard->gameState() == 1)
		{
			gameBoard->stopGame();
			gameBoard->fullRepaint();
		}
		if(gameBoard2->gameState() == 1)
		{
			gameBoard2->stopGame();
			gameBoard2->fullRepaint();
		}
		gameBoard->showMessage(tr("Waiting for other player"));
		gameBoard2->showMessage(tr("Waiting for other player"));
	}
	*/
}

MainWindow::~MainWindow()
{
	delete ui;
}


//private slots


void MainWindow::about()
{
	pauseGame();

	QMessageBox::information(this, tr("About") + " Tubularix 0.4.0.7",
		tr("Tubularix is a game similar to Tetris seen from "
		"a tubular perspective. Enjoy it and send comments, bug reports, or whatever you want to "
		"miguelescudero@users.sourceforge.net.\n\n"
		"Main game controls: P to pause, R to restart game, F to switch between normal mode/full screen "
		"and ESC to exit full screen mode.\n\n"
		"Game controls for player 1 (right player): left and right arrows to move, down arrow to move "
		"down faster, B to drop the piece, up arrow to rotate piece counter-clockwise and space to rotate "
		"clockwise.\n\n"
		"Game controls for player 2 (left player): A and D to move, S to move down faster, X to drop "
		"the piece, W to rotate piece counter-clockwise and Q to rotate clockwise.\n\n"
		"You can find more info about the game in the README file inside the game's dir, or you can go "
		"to the project page http://tubularix.sourceforge.net to find info, updates, etc."));
}

void MainWindow::aboutQt()
{
	pauseGame();
	qApp->aboutQt();
}

void MainWindow::selectGameType()
{
	if(isFullScreen())
		setNormalScreen();

	pauseGame();

	GameTypeDialog dialog(this, currentGameType, currentGameMode, customMode, rows, columns, startLevel,
						  samePieces, sendDestroyedLines);
	if(dialog.exec())
	{
		//if(gameBoard->gameState() != 0)
			//gameBoard->stopGame();
		setGameType(dialog.selectedGameType());
		setGameMode(dialog.selectedGameMode());

		samePieces = dialog.samePieces();
		setSendDestroyedLines(dialog.sendDestroyedLines());

		if(dialog.customMode())
		{
			customMode = true;
			rows = dialog.rows();
			columns = dialog.columns();
		}
		else
		{
			customMode = false;
			switch(dialog.selectedGameMode())
			{
				case 1:
				case 2:
					rows = 18;
					columns = 12;
					break;
				case 3:
				case 4:
				case 5:
				case 6:
					rows = 23;
					columns = 10;
					break;
			}
		}
		gameBoard->setRows(rows);
		gameBoard->setColumns(columns);
		gameBoard2->setRows(rows);
		gameBoard2->setColumns(columns);

		setStartLevel(dialog.startLevel());

		startGame();
	}
}

void MainWindow::showGameOptions()
{
	pauseGame();

	GameOptionsDialog dialog(this);
	if(dialog.exec())
	{
		readGameSettings();

		//Apply some key repetition settings
		keyFirstTimerPlayer1->setInterval(keyRepetitionDelay);
		keyFirstTimerPlayer2->setInterval(keyRepetitionDelay);
		keyRepeatTimerPlayer1->setInterval(1000 / keyRepetitionRate);
		keyRepeatTimerPlayer2->setInterval(1000 / keyRepetitionRate);
	}
}

void MainWindow::setGameType(int value)
{
	//if(currentGameType == value)
		//return;

	if(currentGameType == 1)
		onePlayerWindowGeometry = geometry();
	else if(currentGameType == 2)
		twoPlayersWindowGeometry = geometry();
	/*
	else if(currentGameType == 3)
		twoPlayersWindowGeometry = geometry();
	else if(currentGameType == 4)
		twoPlayersWindowGeometry = geometry();
	*/

	currentGameType = value;
	emit gameTypeChanged(value);

	if(currentGameType == 1)
	{
		setGeometry(onePlayerWindowGeometry);
		gameBoard2->setNetworkedGameboard(false);
	}
	else if(currentGameType == 2)
	{
		setGeometry(twoPlayersWindowGeometry);
		gameBoard2->setNetworkedGameboard(false);
	}
	/*
	else if(currentGameType == 3)
	{
		setGeometry(twoPlayersWindowGeometry);
		gameBoard2->setNetworkedGameboard(false);
		readNetworkServerSettings();
	}
	else if(currentGameType == 4)
	{
		setGeometry(twoPlayersWindowGeometry);
		gameBoard2->setNetworkedGameboard(false);
		readNetworkClientSettings();
	}
	*/
}

void MainWindow::setGameMode(int value)
{
	//if(currentGameMode == value)
		//return;

	currentGameMode = value;
	emit gameModeChanged(value);
}

void MainWindow::setStartLevel(int value)
{
	startLevel = value;
	emit startLevelChanged(value);
}

void MainWindow::readSettings()
{
	QSettings settings;
	settings.beginGroup("MainWindow");
	ui->actionActivateAntialiasing->setChecked(settings.value("antialiasing", true).toBool());
	ui->actionShowMap->setChecked(settings.value("showMap", true).toBool());
	ui->actionPiecesAppearingGradually->setChecked(settings.value("piecesAppearingGradually", true).toBool());
	ui->actionShowGhostPiece->setChecked(settings.value("showGhostPiece", true).toBool());
	setGameType(settings.value("gameType", 1).toInt());
	setGameMode(settings.value("gameMode", 1).toInt());

	//Custom game settings
	customMode = settings.value("customMode", false).toBool();
	rows = settings.value("rows", 18).toInt();
	columns = settings.value("columns", 12).toInt();
	gameBoard->setRows(rows);
	gameBoard->setColumns(columns);
	gameBoard2->setRows(rows);
	gameBoard2->setColumns(columns);

	samePieces = settings.value("samePieces", true).toBool();
	setSendDestroyedLines(settings.value("sendDestroyedLines", true).toBool());

	setStartLevel(settings.value("startLevel", 1).toInt());

	onePlayerWindowGeometry = settings.value("onePlayerWindowGeometry", QRect(0, 0, 530, 580)).toRect();
	twoPlayersWindowGeometry = settings.value("twoPlayersWindowGeometry", QRect(0, 0, 780, 460)).toRect();

	if(currentGameType == 1)
		setGeometry(onePlayerWindowGeometry);
	else if(currentGameType == 2)
		setGeometry(twoPlayersWindowGeometry);
	else if(currentGameType == 3)
		setGeometry(twoPlayersWindowGeometry);
	else if(currentGameType == 4)
		setGeometry(twoPlayersWindowGeometry);
	settings.endGroup();

	settings.beginGroup("General");
	language = settings.value("language", QLocale::system().name()).toString();
	settings.endGroup();

	readGameSettings();
}

void MainWindow::readGameSettings()
{
	QSettings settings;
	settings.beginGroup("GameOptions");

	//Controls
	pauseKey = settings.value("pauseCode", Qt::Key_P).toInt();
	restartKey = settings.value("restartCode", Qt::Key_R).toInt();
	left1Key = settings.value("left1Code", Qt::Key_Left).toInt();
	right1Key = settings.value("rigt1Code", Qt::Key_Right).toInt();
	down1Key = settings.value("down1Code", Qt::Key_Down).toInt();
	drop1Key = settings.value("drop1Code", Qt::Key_B).toInt();
	rotateCC1Key = settings.value("rotateCC1Code", Qt::Key_Up).toInt();
	rotateC1Key = settings.value("rotateC1Code", Qt::Key_Space).toInt();
	left2Key = settings.value("left2Code", Qt::Key_A).toInt();
	right2Key = settings.value("right2Code", Qt::Key_D).toInt();
	down2Key = settings.value("down2Code", Qt::Key_S).toInt();
	drop2Key = settings.value("drop2Code", Qt::Key_X).toInt();
	rotateCC2Key = settings.value("rotateCC2Code", Qt::Key_W).toInt();
	rotateC2Key = settings.value("rotateC2Code", Qt::Key_Q).toInt();

	//Rotation style
	setRotationStyle(settings.value("rotationStyle", 0).toInt());

	//Key repetition
	movementKeyRepetition = settings.value("activateMovementKeyRepetition", true).toBool();
	rotationKeyRepetition = settings.value("activateRotationKeyRepetition", false).toBool();
	keyRepetitionDelay = settings.value("repetitionDelay", 205).toInt();
	keyRepetitionRate = settings.value("repetitionRate", 25).toInt();
	settings.endGroup();

	settings.beginGroup("Colors");
	colors[0].setNamedColor(settings.value("color0", QColor(90, 90, 90).name()).toString());
	colors[1].setNamedColor(settings.value("color1", QColor(250, 70, 70).name()).toString());
	colors[2].setNamedColor(settings.value("color2", QColor(60, 60, 250).name()).toString());
	colors[3].setNamedColor(settings.value("color3", QColor(230, 240, 40).name()).toString());
	colors[4].setNamedColor(settings.value("color4", QColor(50, 180, 240).name()).toString());
	colors[5].setNamedColor(settings.value("color5", QColor(130, 20, 250).name()).toString());
	colors[6].setNamedColor(settings.value("color6", QColor(250, 180, 60).name()).toString());
	colors[7].setNamedColor(settings.value("color7", QColor(140, 240, 40).name()).toString());
	colors[8].setNamedColor(settings.value("color8", QColor(70, 70, 70).name()).toString());
	settings.endGroup();

	emit colorsChanged(colors);
}

/*
void MainWindow::readNetworkClientSettings()
{
	QSettings settings;
	settings.beginGroup("Network Cient");
	ip = settings.value("ip", "127.0.0.1").toString();
	port = settings.value("port", 5124).toInt();
	settings.endGroup();

	tcpClient.connectToHost(ip, port);
}
*/

/*
void MainWindow::readNetworkServerSettings()
{
	QSettings settings;
	settings.beginGroup("Network Server");
	ip = settings.value("ip", "127.0.0.1").toString();
	port = settings.value("port", 5124).toInt();
	settings.endGroup();

	if(!tcpServer.listen(QHostAddress(ip), port))
	{
		cout<<endl<<"Error: server unable to connect"<<endl;

		//just for testing purposes!!!
		currentGameType = 4;
		emit gameTypeChanged(4);
		tcpClient.connectToHost(ip, port);
	}
}
*/

void MainWindow::writeSettings()
{
	QSettings settings;
	settings.beginGroup("MainWindow");
	settings.setValue("antialiasing", ui->actionActivateAntialiasing->isChecked());
	settings.setValue("showMap", ui->actionShowMap->isChecked());
	settings.setValue("piecesAppearingGradually", ui->actionPiecesAppearingGradually->isChecked());
	settings.setValue("showGhostPiece", ui->actionShowGhostPiece->isChecked());
	settings.setValue("gameType", currentGameType);
	settings.setValue("gameMode", currentGameMode);

	//Custom game settings
	settings.setValue("customMode", customMode);
	settings.setValue("rows", rows);
	settings.setValue("columns", columns);

	settings.setValue("samePieces", samePieces);
	settings.setValue("sendDestroyedLines", sendDestroyedLines);

	settings.setValue("startLevel", startLevel);

	settings.setValue("onePlayerWindowGeometry", onePlayerWindowGeometry);
	settings.setValue("twoPlayersWindowGeometry", twoPlayersWindowGeometry);
	settings.endGroup();

	settings.beginGroup("General");
	settings.setValue("language", language);
	settings.endGroup();
}

void MainWindow::selectLanguage()
{
	pauseGame();

	bool ok;
	QStringList availableLanguages;
	availableLanguages<<tr("cs (czech)")<<tr("en (english)")<<tr("es (spanish)")<<tr("gl (galician)");

	QString selectedLanguage = QInputDialog::getItem(this, tr("Language selection"),
		tr("Select application's language (you must restart before changes will take effect)"),
		availableLanguages, 0, false, &ok);

	if(ok && !selectedLanguage.isEmpty())
	{
		language = selectedLanguage.left(2);
		QMessageBox::information(this, tr("Language selection"),
			tr("You must restart the application before changes will take effect"));
	}
}

void MainWindow::restoreSizeSettings()
{
	if(isFullScreen())
		setNormalScreen();

	if(currentGameType == 1)
		setGeometry(200, 100, 1024, 900);
	else if(currentGameType == 2)
		setGeometry(200, 100, 1024, 518);
}

void MainWindow::changeFullScreen()
{
	if(isFullScreen())
		setNormalScreen();
	else if(isMaximized())
	{
		showNormal();
		showFullScreen();
		//ui->scrollArea->setFrameShape(QFrame::NoFrame);
		splitter->setFrameShape(QFrame::NoFrame);
		//ui->scrollArea->layout()->setMargin(0);
		hideExtraElements();
	}
	else
	{
		showFullScreen();
		//ui->scrollArea->setFrameShape(QFrame::NoFrame);
		splitter->setFrameShape(QFrame::NoFrame);
		//ui->scrollArea->layout()->setMargin(0);
		hideExtraElements();
	}
}

void MainWindow::setNormalScreen()
{
	showNormal();
	//ui->scrollArea->setFrameShape(QFrame::StyledPanel);
	splitter->setFrameShape(QFrame::StyledPanel);
	//ui->scrollArea->layout()->setMargin(2);
	showExtraElements();
}

void MainWindow::showExtraElements()
{
	if(!ui->mainToolBar->isVisible() || !ui->menuBar->isVisible())
	{
		ui->mainToolBar->show();
		ui->menuBar->show();
	}
}

void MainWindow::hideExtraElements()
{
	if(isFullScreen() && (ui->mainToolBar->isVisible() && ui->menuBar->isVisible()))
	{
		ui->mainToolBar->hide();
		ui->menuBar->hide();
	}

}

void MainWindow::startGame()
{
	//qsrand(1);
	pieces.clear();
	if(samePieces) {
		for(int i = 0; i < 10; i++)
			pieces.append(qrand() % 7);
	}
	gameBoard->setNextPiecesList(pieces);
	if(currentGameType >= 2)
		gameBoard2->setNextPiecesList(pieces);

	keyFirstEventList[0] = -1;
	keyRepeatEventList[0] = -1;
	keyRepeatTimerPlayer1->start();
	if(currentGameType >= 2)
	{
		keyFirstEventList[1] = -1;
		keyRepeatEventList[1] = -1;
		keyRepeatTimerPlayer2->start();
	}

	if(currentGameType == 1) //start one player game
	{
		gameBoard->startGame();
		gameBoard2->hide();
	}
	else if(currentGameType >= 2) //start two players game
	{
		gameBoard->startGame();
		gameBoard2->startGame();
		gameBoard2->show();
	}
}

void MainWindow::pauseGame()
{
	//player 1
	if(gameBoard->gameState() == 1)
		gameBoard->pauseGame(true);

	//player 2
	if(currentGameType >= 2 && gameBoard2->gameState() == 1)
		gameBoard2->pauseGame(true);
}

void MainWindow::endGame()
{
	//one player game
	if(currentGameType == 1)
	{
		keyFirstTimerPlayer1->stop();
		keyRepeatTimerPlayer1->stop();

		gameBoard->showMessage(tr("game over"), tr("press r to restart"));

		int pos = checkHighScore();
		if(pos == 0)
		{
			if(ui->actionActivateAiPlayer1->isChecked())
				QTimer::singleShot(4000, this, SLOT(startGame()));
			return;
		}

		bool ok;
		playerName = QInputDialog::getText(this, tr("High scores"),
		   tr("Enter your name"), QLineEdit::Normal, playerName, &ok);
		playerName.left(10);
		if(ok && !playerName.isEmpty())
		{
			writeHighScores();
			showHighScores(pos);
		}
	}

	//if two player game selected, just show a message telling who is the winner. Don't show high scores.
	if(currentGameType >= 2)
	{
		if(sendDestroyedLines)
		{
			keyFirstTimerPlayer1->stop();
			keyRepeatTimerPlayer1->stop();
			keyFirstTimerPlayer2->stop();
			keyRepeatTimerPlayer2->stop();

			if(gameBoard->gameState() == 0 && gameBoard2->gameState() == 1)
			{
				gameBoard2->addTotalGames(1);
				gameBoard->addTotalScore(gameBoard->score());
				gameBoard2->addTotalScore(gameBoard2->score());

				gameBoard2->stopGame();

				gameBoard->showMessage(tr("you lose!"), tr("press r to restart"));
				gameBoard2->showMessage(tr("you win!"), tr("press r to restart"));
				if(ui->actionActivateAiPlayer1->isChecked() && ui->actionActivateAiPlayer2->isChecked())
					QTimer::singleShot(4000, this, SLOT(startGame()));
			}
			else if(gameBoard2->gameState() == 0 && gameBoard->gameState() == 1)
			{
				gameBoard->addTotalGames(1);
				gameBoard->addTotalScore(gameBoard->score());
				gameBoard2->addTotalScore(gameBoard2->score());

				gameBoard->stopGame();

				gameBoard2->showMessage(tr("you lose!"), tr("press r to restart"));
				gameBoard->showMessage(tr("you win!"), tr("press r to restart"));
				if(ui->actionActivateAiPlayer1->isChecked() && ui->actionActivateAiPlayer2->isChecked())
					QTimer::singleShot(4000, this, SLOT(startGame()));
			}
			/*else
			{
				QMessageBox::information(this, tr("Two player game"), tr("Draw: no one wins"));
			}*/
		}
		else
		{
			if(gameBoard->gameState() == 0 && gameBoard2->gameState() == 1)
			{
				gameBoard->showMessage(tr("game over"), tr("wait until the other player ends"));
				return;
			}
			if(gameBoard2->gameState() == 0 && gameBoard->gameState() == 1)
			{
				gameBoard2->showMessage(tr("game over"), tr("wait until the other player ends"));
				return;
			}

			keyFirstTimerPlayer1->stop();
			keyRepeatTimerPlayer1->stop();
			keyFirstTimerPlayer2->stop();
			keyRepeatTimerPlayer2->stop();

			gameBoard->addTotalScore(gameBoard->score());
			gameBoard2->addTotalScore(gameBoard2->score());

			//compare scores (gameBoard number is the oposite of Player number)
			QString message;
			if(gameBoard->score() > gameBoard2->score())
			{
				gameBoard->addTotalGames(1);
				message = tr("Right player wins with a difference of ")
							 + QString::number(gameBoard->score() - gameBoard2->score()) + tr(" points");
			}
			else if(gameBoard->score() < gameBoard2->score())
			{
				gameBoard2->addTotalGames(1);
				message = tr("Left player wins with a difference of ")
							 + QString::number(gameBoard2->score() - gameBoard->score()) + tr(" points");
			}
			else
				message = tr("Draw: no one wins");

			gameBoard->showMessage(tr("game over"), tr("press r to restart"));
			gameBoard2->showMessage(tr("game over"), tr("press r to restart"));

			//show result and exit
			QMessageBox::information(this, tr("Two player game"), message);
		}
	}
}

void MainWindow::showHighScores(int winner)
{
	pauseGame();

	QList<QStringList> stringList = readHighScores();
	QString message;
	message.append("<font family=\"monospace\" size=3><b>");
	message.append("<pre STYLE=\"font-family:'monospace'\"><u>Pos Score           "
		"Level   Lines     Name            Hour      Date      </u></pre>");
	for(int i = 0; i < stringList.size(); i++)
	{
		if(i == winner - 1)
			message.append("<pre><font color=red>" + QString::number(winner).rightJustified(3, ' ')
				+ " " + stringList[i].join("  ") + "</font></pre>");
		else if(!stringList.at(i).isEmpty())
			message.append("<pre>" + QString::number(i + 1).rightJustified(3, ' ')
				+ " " + stringList[i].join("  ") + "\n" + "</pre>");
	}
	message.append("</b></font>");
	QMessageBox messageBox;
	messageBox.setTextFormat(Qt::RichText);
	messageBox.setWindowTitle(tr("Tubularix - High Scores"));
	messageBox.setText(message);
	messageBox.exec();
}

int MainWindow::checkHighScore()
{
	QList<QStringList> stringList = readHighScores();

	for(int i = 0; i < 10; i++)
	{
		if(stringList.at(i).isEmpty())
			return i + 1;
		if(gameBoard->score() > stringList.at(i).at(0).toInt())
			return i + 1;
	}

	return 0;
}

QList<QStringList> MainWindow::readHighScores()
{
	QList<QStringList> stringList;

	QSettings settings;
	if(customMode)
		settings.beginGroup("HighScoresCustom" + QString::number(currentGameMode));
	else
		settings.beginGroup("HighScores" + QString::number(currentGameMode));

	for(int i = 0; i < 10; i++)
	{
		stringList.append(settings.value("score" + QString::number(i + 1), QStringList()).toStringList());
	}
	settings.endGroup();

	return stringList;
}

void MainWindow::writeHighScores()
{
	QList<QStringList> stringList = readHighScores();

	//set currentScoreData values
	QStringList currentScoreData;
	currentScoreData<<QString::number(gameBoard->score()).leftJustified(14, ' ');
	currentScoreData<<QString::number(gameBoard->level()).leftJustified(6, ' ' );
	currentScoreData<<QString::number(gameBoard->lines()).leftJustified(8, ' ' );
	currentScoreData<<playerName.leftJustified(14, ' ');
	currentScoreData<<QTime::currentTime().toString("hh:mm:ss");
	currentScoreData<<QDate::currentDate().toString("yyyy-MM-dd");

	for(int i = 0; i < 10; i++)
	{
		if(stringList.at(i).isEmpty())
		{
			stringList.replace(i, currentScoreData);
			break;
		}
		else if(currentScoreData.at(0).toInt() > stringList.at(i).at(0).toInt())
		{
			stringList.insert(i, currentScoreData);
			break;
		}
	}

	int stringListSize = 10;
	if (stringList.size() < 10)
		stringListSize = stringList.size();


	QSettings settings;
	if(customMode)
		settings.beginGroup("HighScoresCustom" + QString::number(currentGameMode));
	else
		settings.beginGroup("HighScores" + QString::number(currentGameMode));

	//that may solve some QSetting problems in OS/2
	/*
	for(int i = 0; i < stringListSize; i++)
	{
		for(int j = 0; j < stringList[i].size(); j++)
			stringList[i][j] = stringList[i][j].toLatin1();
	}
	*/

	for(int i = 0; i < stringListSize; i++)
		settings.setValue("score" + QString::number(i + 1), stringList[i]);
	settings.endGroup();
}

void MainWindow::updateFirstKeyPressPlayer1()
{
	if(gameBoard->gameState() != 1 || keyFirstEventList[0] == -1)
		return;

	if(keyFirstEventList[0] == left1Key)
	{
		gameBoard->moveLeft();
		if(movementKeyRepetition)
			keyFirstTimerPlayer1->start();
	}
	if(keyFirstEventList[0] == right1Key)
	{
		gameBoard->moveRight();
		if(movementKeyRepetition)
			keyFirstTimerPlayer1->start();
	}
	if(keyFirstEventList[0] == rotateCC1Key)
	{
		gameBoard->rotatePieceCounterClockwise();
		if(rotationKeyRepetition)
			keyFirstTimerPlayer1->start();
	}
	if(keyFirstEventList[0] == rotateC1Key)
	{
		gameBoard->rotatePieceClockwise();
		if(rotationKeyRepetition)
			keyFirstTimerPlayer1->start();
	}
}

void MainWindow::updateFirstKeyPressPlayer2()
{
	if(currentGameType < 2 || gameBoard2->gameState() != 1 || keyFirstEventList[1] == -1)
		return;

	if(keyFirstEventList[1] == left2Key)
	{
		gameBoard2->moveLeft();
		if(movementKeyRepetition)
			keyFirstTimerPlayer2->start();
	}
	if(keyFirstEventList[1] == right2Key)
	{
		gameBoard2->moveRight();
		if(movementKeyRepetition)
			keyFirstTimerPlayer2->start();
	}
	if(keyFirstEventList[1] == rotateCC2Key)
	{
		gameBoard2->rotatePieceCounterClockwise();
		if(rotationKeyRepetition)
			keyFirstTimerPlayer2->start();
	}
	if(keyFirstEventList[1] == rotateC2Key)
	{
		gameBoard2->rotatePieceClockwise();
		if(rotationKeyRepetition)
			keyFirstTimerPlayer2->start();
	}
}

void MainWindow::updateRepeatKeyPressPlayer1()
{
	if(gameBoard->gameState() != 1 || keyRepeatEventList[0] == -1 || keyRepeatEventList[0] != keyFirstEventList[0])
		return;

	if(keyRepeatEventList[0] == left1Key)
	{
		if(movementKeyRepetition)
			gameBoard->moveLeft();
	}
	if(keyRepeatEventList[0] == right1Key)
	{
		if(movementKeyRepetition)
			gameBoard->moveRight();
	}
	if(keyRepeatEventList[0] == rotateCC1Key)
	{
		if(rotationKeyRepetition)
			gameBoard->rotatePieceCounterClockwise();
	}
	if(keyRepeatEventList[0] == rotateC1Key)
	{
		if(rotationKeyRepetition)
			gameBoard->rotatePieceClockwise();
	}
	
	keyFirstTimerPlayer1->stop();
}

void MainWindow::updateRepeatKeyPressPlayer2()
{
	if(currentGameType < 2 || gameBoard2->gameState() != 1 || keyRepeatEventList[1] == -1 ||
	   keyRepeatEventList[1] != keyFirstEventList[1])
		return;

	if(keyRepeatEventList[1] == left2Key)
	{
		if(movementKeyRepetition)
			gameBoard2->moveLeft();
	}
	if(keyRepeatEventList[1] == right2Key)
	{
		if(movementKeyRepetition)
			gameBoard2->moveRight();
	}
	if(keyRepeatEventList[1] == rotateCC2Key)
	{
		if(rotationKeyRepetition)
			gameBoard2->rotatePieceCounterClockwise();
	}
	if(keyRepeatEventList[1] == rotateC2Key)
	{
		if(rotationKeyRepetition)
			gameBoard2->rotatePieceClockwise();
	}
	
	keyFirstTimerPlayer2->stop();
}

void MainWindow::setRepeatKeyPlayer1()
{
	keyRepeatEventList[0] = keyFirstEventList[0];
}

void MainWindow::setRepeatKeyPlayer2()
{
	keyRepeatEventList[1] = keyFirstEventList[1];
}

void MainWindow::setAiPlayer1(bool value)
{
	if(value == true)
		aiMovementTimer1->start();
	else
		aiMovementTimer1->stop();
}

void MainWindow::setAiPlayer2(bool value)
{
	if(value == true)
		aiMovementTimer2->start();
	else
		aiMovementTimer2->stop();
}

//very bad ai movement, and absolutely messy code, but useful for test purposes
void MainWindow::aiMovement1()
{
	if(gameBoard->gameState() != 1)
		return;

	//crea una lista con las posiciones más bajas y otra con las que menos cuadrados dejan libres,
	//busca coincidencias y selecciona aleatoriamente una de las posiciones si hay varias
	//igual de buenas o si no hay coincidencias
	/*
	QMap <int, int> lowPositionsDownside;
	QMap <int, int> lowPositionsUpside;
	QMap <int, int> lessEmptySquaresInmediately;
	QMap <int, int> lessEmptySquares;
	*/

	//Criterios:
	//llega más abajo
	//gameBoard->calculateFallPosition(gameBoard->piece()->posX(), gameBoard->piece()->posY())
	//	+ gameBoard->piece()->firstDown();

	//queda menos elevada
	//gameBoard->calculateFallPosition(gameBoard->piece()->posX(), gameBoard->piece()->posY())
	//  + gameBoard->piece()->firstDown();// - gameBoard->piece()->height();

	//deja menos huecos inmediantamente debajo o tal vez mejor hasta encontrar una superficie
	//crea una función similar a la otra

	//deja menos huecos debajo de ella
	//gameBoard->calculateEmptySquaresUnderDroppedPiece(gameBoard->piece()->posX(),
	//	gameBoard->piece()->posY());

	//calculate the best position
	int pos = gameBoard->calculateFallPosition(gameBoard->piece()->posX(), gameBoard->piece()->posY())
			  + gameBoard->piece()->firstDown();// - gameBoard->piece()->height();
	int col = gameBoard->piece()->posX();
	int rot = 0;
	static int emptySquares = -1;
	static int pieceFirstUp = -1;

	if(emptySquares == -1)
	{
		emptySquares = gameBoard->calculateEmptySquaresUnderDroppedPiece(gameBoard->piece()->posX(),
			gameBoard->piece()->posY());
	}
	if(pieceFirstUp == -1)
		pieceFirstUp = gameBoard->piece()->firstUp();
	for(int i = 0; i < 4; i++)
	{
		for(int j = 0; j < gameBoard->totalColumns(); j++)
		{
			int eS = gameBoard->calculateEmptySquaresUnderDroppedPiece(j, gameBoard->piece()->posY());
			int p = gameBoard->calculateFallPosition(j, gameBoard->piece()->posY())
				+ gameBoard->piece()->firstDown();// - gameBoard->piece()->height();
			/*if(eS < emptySquares && eS >= 0)
			{
				pos = p;
				col = j;
				rot = i;
				emptySquares = eS;
			}
			else if(eS == emptySquares && eS >= 0 && p > pos)
			{
				pos = p;
				col = j;
				rot = i;
				emptySquares = eS;
			}*/
			if(p > pos)
			{
				pos = p;
				col = j;
				rot = i;
				emptySquares = eS;
			}
			if(p == pos && eS < emptySquares && eS >= 0)
			{
				pos = p;
				col = j;
				rot = i;
				emptySquares = eS;
			}
			/*else if(p == pos && eS == emptySquares && eS >= 0 && gameBoard->piece()->firstUp() > pieceFirstUp)
			{
				pos = p;
				col = j;
				rot = i;
				emptySquares = eS;
			}*/
		}
		gameBoard->piece()->rotateCounterClockwise();
	}

	//move the piece to the previously selected as best position
	for(int i = 0; i < rot; i++)
		gameBoard->rotatePieceCounterClockwise();

	if(gameBoard->piece()->posX() > col)
		gameBoard->moveLeft();
	else if(gameBoard->piece()->posX() < col)
		gameBoard->moveRight();
	else if(!rot)
		gameBoard->speedUp();

	if(!rot)
	{
		emptySquares = -1;
		pieceFirstUp = -1;
	}
}

//very bad ai movement, and absolutely messy code, but useful for test purposes
void MainWindow::aiMovement2()
{
	if(gameBoard2->gameState() != 1)
		return;

	//crea una lista con las posiciones más bajas y otra con las que menos cuadrados dejan libres,
	//busca coincidencias y selecciona aleatoriamente una de las posiciones si hay varias
	//igual de buenas o si no hay coincidencias
	/*
	QMap <int, int> lowPositionsDownside;
	QMap <int, int> lowPositionsUpside;
	QMap <int, int> lessEmptySquaresInmediately;
	QMap <int, int> lessEmptySquares;
	*/

	//Criterios:
	//llega más abajo
	//gameBoard->calculateFallPosition(gameBoard->piece()->posX(), gameBoard->piece()->posY())
	//	+ gameBoard->piece()->firstDown();

	//queda menos elevada
	//gameBoard->calculateFallPosition(gameBoard->piece()->posX(), gameBoard->piece()->posY())
	//  + gameBoard->piece()->firstDown();// - gameBoard->piece()->height();

	//deja menos huecos inmediantamente debajo o tal vez mejor hasta encontrar una superficie
	//crea una función similar a la otra

	//deja menos huecos debajo de ella
	//gameBoard->calculateEmptySquaresUnderDroppedPiece(gameBoard->piece()->posX(),
	//	gameBoard->piece()->posY());

	//calculate the best position
	int pos = gameBoard2->calculateFallPosition(gameBoard2->piece()->posX(), gameBoard2->piece()->posY())
			  + gameBoard2->piece()->firstDown();// - gameBoard->piece()->height();
	int col = gameBoard2->piece()->posX();
	int rot = 0;
	static int emptySquares = -1;
	static int pieceFirstUp = -1;

	if(emptySquares == -1)
	{
		emptySquares = gameBoard2->calculateEmptySquaresUnderDroppedPiece(gameBoard2->piece()->posX(),
			gameBoard2->piece()->posY());
	}
	if(pieceFirstUp == -1)
		pieceFirstUp = gameBoard2->piece()->firstUp();
	for(int i = 0; i < 4; i++)
	{
		for(int j = 0; j < gameBoard2->totalColumns(); j++)
		{
			int eS = gameBoard2->calculateEmptySquaresUnderDroppedPiece(j, gameBoard2->piece()->posY());
			int p = gameBoard2->calculateFallPosition(j, gameBoard2->piece()->posY())
				+ gameBoard2->piece()->firstDown();// - gameBoard2->piece()->height();
			/*if(eS < emptySquares && eS >= 0)
			{
				pos = p;
				col = j;
				rot = i;
				emptySquares = eS;
			}
			else if(eS == emptySquares && eS >= 0 && p > pos)
			{
				pos = p;
				col = j;
				rot = i;
				emptySquares = eS;
			}*/
			if(p > pos)
			{
				pos = p;
				col = j;
				rot = i;
				emptySquares = eS;
			}
			if(p == pos && eS < emptySquares && eS >= 0)
			{
				pos = p;
				col = j;
				rot = i;
				emptySquares = eS;
			}
			/*else if(p == pos && eS == emptySquares && eS >= 0 && gameBoard->piece()->firstUp() > pieceFirstUp)
			{
				pos = p;
				col = j;
				rot = i;
				emptySquares = eS;
			}*/
		}
		gameBoard2->piece()->rotateCounterClockwise();
	}

	//move the piece to the previously selected as best position
	for(int i = 0; i < rot; i++)
		gameBoard2->rotatePieceCounterClockwise();

	if(gameBoard2->piece()->posX() > col)
		gameBoard2->moveLeft();
	else if(gameBoard2->piece()->posX() < col)
		gameBoard2->moveRight();
	else if(!rot)
		gameBoard2->speedUp();

	if(!rot)
	{
		emptySquares = -1;
		pieceFirstUp = -1;
	}
}

/*
void MainWindow::acceptConnection()
{
	tcpServerConnection = tcpServer.nextPendingConnection();
	connect(tcpServerConnection, SIGNAL(readyRead()), this, SLOT(processServerPendingStreams()));
	//connect(tcpServerConnection, SIGNAL(error(QAbstractSocket::SocketError)),
		 //this, SLOT(displayError(QAbstractSocket::SocketError)));
	tcpServer.close();

	//send data from server to client:
	//serverRandomSeed, serverRotationKeyRepetition, serverMovementKeyRepetition
	startGame();
}
*/

/*
void MainWindow::processClientPendingStreams()
{
	static int n = 0;
	QDataStream in(&tcpClient);
	in.setVersion(QDataStream::Qt_4_4);

	//cout<<"Comenzando recepción de captura de pantalla ("<<nextClientBlockSize<<" bytes) "<<endl;

	while(1)
	{
		if(nextClientBlockSize == 0)
		{
			if(tcpClient.bytesAvailable() < (int)sizeof(quint32))
				break;
			in>>nextClientBlockSize;
		}

		//cout<<"Comenzando recepción de captura de pantalla ("
		//cout<<nextClientBlockSize<<" bytes) (paso 2)"<<endl;
		//cout<<"Bytes available = "<<tcpClient.bytesAvailable()<<endl;

		if(tcpClient.bytesAvailable() < nextClientBlockSize)
			break;

		//QPixmap pixmap;
		//in>>pixmap;
		//gameBoard2->setPixmap(pixmap);
		QString keyName;
		int type;
		bool autorep;
		in>>keyName;
		in>>type;
		in>>autorep;
		processKeyStream(keyName, type, autorep);

		nextClientBlockSize = 0;

		cout<<"Envío de datos número "<<++n<<" recibido"<<endl;
	}
}
*/

/*
void MainWindow::processServerPendingStreams()
{
	static int n = 0;
	QDataStream in(tcpServerConnection);
	in.setVersion(QDataStream::Qt_4_4);

	//cout<<"Comenzando recepción de captura de pantalla"<<nextServerBlockSize<<" bytes) "<<endl;

	while(1)
	{
		if(nextServerBlockSize == 0)
		{
			if(tcpServerConnection->bytesAvailable() < (int)sizeof(quint32))
				break;
			in>>nextServerBlockSize;
		}

		//cout<<"Comenzando recepción de captura de pantalla ("
		//cout<<nextServerBlockSize<<" bytes) (paso 2)"<<endl;
		//cout<<"Bytes available = "<<tcpServerConnection->bytesAvailable()<<endl;

		if(tcpServerConnection->bytesAvailable() < nextServerBlockSize)
			break;

		//QPixmap pixmap;
		//in>>pixmap;
		//gameBoard2->setPixmap(pixmap);
		QString keyName;
		int type;
		bool autorep;
		in>>keyName;
		in>>type;
		in>>autorep;
		processKeyStream(keyName, type, autorep);

		nextServerBlockSize = 0;

		cout<<"Envío de datos número "<<++n<<" recibido"<<endl;
	}
}
*/

/*
void MainWindow::processKeyStream(QString keyName, int type, bool autorep)
{
	//pause/continue game
	if(keyName == "Pause" && !autorep)
	{
		//player 1
		if(gameBoard->gameState() == 1)
			gameBoard->pauseGame(true);
		else if(gameBoard->gameState() == 2)
			gameBoard->pauseGame(false);

		//player 2
		if(currentGameType >= 2)
		{
			if(gameBoard2->gameState() == 1)
				gameBoard2->pauseGame(true);
			else if(gameBoard2->gameState() == 2)
				gameBoard2->pauseGame(false);
		}
	}

	//restart game
	if(keyName == "Restart")
		startGame();

	if(keyName == "Left" || keyName == "Right" || keyName == "Down" || keyName == "RotateCC" ||
		keyName == "RotateC")
	{
		int key = 0;

		if(keyName == "Left")
			key = left2Key;
		else if(keyName == "Right")
			key = right2Key;
		else if(keyName == "Down")
			key = down2Key;
		else if(keyName == "RotateCC")
			key = rotateCC2Key;
		else if(keyName == "RotateC")
			key = rotateC2Key;

		QKeyEvent event(QEvent::Type(type), key, Qt::NoModifier, "", autorep);
		if(QEvent::Type(type) == QEvent::KeyPress)
			keyPressEvent(&event);
		else if(QEvent::Type(type) == QEvent::KeyRelease)
			keyReleaseEvent(&event);
	}
}
*/

/*
void MainWindow::sendClientStream(QString keyName, int type, bool autorep)
{
	static int n = 0;
	QByteArray data;
	QDataStream out(&data, QIODevice::WriteOnly);
	out.setVersion(QDataStream::Qt_4_4);
	out<<quint32(0);
	//out<<gameBoard->screenShot();
	out<<keyName;
	out<<type;
	out<<autorep;
	out.device()->seek(0);
	out<<quint32(data.size() - (int)sizeof(quint32));
	tcpClient.write(data);

	cout<<"Cliente envía captura número "<<++n<<", tamaño: "<<quint32(data.size() - (int)sizeof(quint32))<<endl;
}
*/

/*
void MainWindow::sendServerStream(QString keyName, int type, bool autorep)
{
	static int n = 0;
	QByteArray data;
	QDataStream out(&data, QIODevice::WriteOnly);
	out.setVersion(QDataStream::Qt_4_4);
	out<<quint32(0);
	//out<<gameBoard->screenShot();
	out<<keyName;
	out<<type;
	out<<autorep;
	out.device()->seek(0);
	out<<quint32(data.size() - (int)sizeof(quint32));
	tcpServerConnection->write(data);

	cout<<"Servidor envía captura número "<<++n<<", tamaño: "<<quint32(data.size() - (int)sizeof(quint32))<<endl;
}
*/

/*
void MainWindow::closeConnection()
{
	tcpClient.close();
}
*/

void MainWindow::addPiece()
{
	if(samePieces) {
		pieces.append(qrand() % 7);
		gameBoard->addNextPiece(pieces.last());
		gameBoard2->addNextPiece(pieces.last());
	}
}

void MainWindow::setSendDestroyedLines(bool value)
{
	sendDestroyedLines = value;
	if(value)
	{
		connect(gameBoard, SIGNAL(linesDestroyed(int)), gameBoard2, SLOT(increaseLinesToAdd(int)));
		connect(gameBoard2, SIGNAL(linesDestroyed(int)), gameBoard, SLOT(increaseLinesToAdd(int)));
	}
	else
	{
		disconnect(gameBoard, SIGNAL(linesDestroyed(int)), gameBoard2, SLOT(increaseLinesToAdd(int)));
		disconnect(gameBoard2, SIGNAL(linesDestroyed(int)), gameBoard, SLOT(increaseLinesToAdd(int)));
	}
}

void MainWindow::setRotationStyle(int style)
{
	rotationStyle = style;
	emit rotationStyleChanged(style);
}

//protected


void MainWindow::closeEvent(QCloseEvent *event)
{
	writeSettings();
	//qApp->quit();
	event->accept();

	QMainWindow::closeEvent(event);
}

void MainWindow::keyPressEvent(QKeyEvent *event)
{
	//main controls
	//pause/continue game
	if(event->key() == pauseKey && !event->isAutoRepeat())
	{
		//player 1
		if(gameBoard->gameState() == 1)
			gameBoard->pauseGame(true);
		else if(gameBoard->gameState() == 2)
			gameBoard->pauseGame(false);

		//player 2
		if(currentGameType >= 2)
		{
			if(gameBoard2->gameState() == 1)
				gameBoard2->pauseGame(true);
			else if(gameBoard2->gameState() == 2)
				gameBoard2->pauseGame(false);
		}
	}

	//restart game
	if(event->key() == restartKey)
		startGame();

	//switch between normal mode/fullScreen
	if(event->key() == Qt::Key_F)
		changeFullScreen();

	//escape fullScreen/maximized and show normal mode
	if(event->key() == Qt::Key_Escape)
		setNormalScreen();

	/*
	//show/hide ghost piece
	if(event->key() == Qt::Key_G)
		ui->actionShowGhostPiece->toggle();
	*/

	/*
	//ai movement for player 1 enable/disable
	if(event->key() == Qt::Key_M)
	{
		if(aiMovementTimer1->isActive())
			aiMovementTimer1->stop();
		else
			aiMovementTimer1->start();
	}
	*/

	/*
	//ai movement for player 2 enable/disable
	if(event->key() == Qt::Key_N)
	{
		if(aiMovementTimer2->isActive())
			aiMovementTimer2->stop();
		else
			aiMovementTimer2->start();
	}
	*/

	//player 1
	if(gameBoard->gameState() == 1 && !event->isAutoRepeat())
	{
		if(event->key() == left1Key || event->key() == right1Key || event->key() == rotateCC1Key ||
		   event->key() == rotateC1Key)
		{
			keyFirstEventList[0] = event->key();
			keyRepeatEventList[0] = -1;
			updateFirstKeyPressPlayer1();
		}
		if(event->key() == down1Key)
			gameBoard->speedUp();
		if(event->key() == drop1Key)
			gameBoard->dropPiece();

		//trick to send a line to the other player
		/*if(event->key() == Qt::Key_L && sendDestroyedLines)
			gameBoard2->increaseLinesToAdd(2);*/
	}

	//player 2
	if(currentGameType >= 2 && gameBoard2->gameState() == 1 && !event->isAutoRepeat())
	{
		if(event->key() == left2Key || event->key() == right2Key || event->key() == rotateCC2Key ||
		   event->key() == rotateC2Key)
		{
			keyFirstEventList[1] = event->key();
			keyRepeatEventList[1] = -1;
			updateFirstKeyPressPlayer2();
		}
		if(event->key() == down2Key)
			gameBoard2->speedUp();
		if(event->key() == drop2Key)
			gameBoard2->dropPiece();
	}

	/*
	//network
	QString keyName;
	if(event->key() == pauseKey)
		keyName = "Pause";
	else if(event->key() == restartKey)
		keyName = "Restart";
	else if(event->key() == left1Key)
		keyName = "Left";
	else if(event->key() == right1Key)
		keyName = "Right";
	else if(event->key() == down1Key)
		keyName = "Down";
	else if(event->key() == rotateCC1Key)
		keyName = "RotateCC";
	else if(event->key() == rotateC1Key)
		keyName = "RotateC";

	if(!keyName.isEmpty())
	{
		if(currentGameType == 3)
			sendServerStream(keyName, event->type(), event->isAutoRepeat());
		else if(currentGameType == 4)
			sendClientStream(keyName, event->type(), event->isAutoRepeat());
	}
	*/

	QMainWindow::keyPressEvent(event);
}

void MainWindow::keyReleaseEvent(QKeyEvent *event)
{
	//player 1
	//if(gameBoard->gameState() == 1)
	{
		if(event->key() == down1Key)
		{
			if(!event->isAutoRepeat())
				gameBoard->restoreSpeed();
		}
		if(event->key() == left1Key || event->key() == right1Key || event->key() == rotateCC1Key ||
		   event->key() == rotateC1Key)
		{
			if(!event->isAutoRepeat() && event->key() == keyFirstEventList[0])
			{
				keyRepeatEventList[0] = -1;
				keyFirstEventList[0] = -1;
			}
		}
	}

	//player 2
	//if(gameBoard2->gameState() == 1)
	if(currentGameType >= 2)
	{
		if(event->key() == down2Key)
		{
			if(!event->isAutoRepeat())
				gameBoard2->restoreSpeed();
		}
		if(event->key() == left2Key || event->key() == right2Key || event->key() == rotateCC2Key ||
		   event->key() == rotateC2Key)
		{
			if(!event->isAutoRepeat() && event->key() == keyFirstEventList[1])
			{
				keyRepeatEventList[1] = -1;
				keyFirstEventList[1] = -1;
			}
		}
	}

	/*
	//network
	QString keyName;
	if(event->key() == left1Key)
		keyName = "Left";
	else if(event->key() == right1Key)
		keyName = "Right";
	else if(event->key() == down1Key)
		keyName = "Down";
	else if(event->key() == rotateCC1Key)
		keyName = "RotateCC";
	else if(event->key() == rotateC1Key)
		keyName = "RotateC";

	if(!keyName.isEmpty())
	{
		if(currentGameType == 3)
			sendServerStream(keyName, event->type(), event->isAutoRepeat());
		else if(currentGameType == 4)
			sendClientStream(keyName, event->type(), event->isAutoRepeat());
	}
	*/

	QMainWindow::keyReleaseEvent(event);
}

void MainWindow::moveEvent(QMoveEvent *event)
{
	if(!isFullScreen() && !isMaximized())
	{
		if(currentGameType == 1)
			onePlayerWindowGeometry = geometry();
		else if(currentGameType == 2)
			twoPlayersWindowGeometry = geometry();
		else if(currentGameType == 3)
			twoPlayersWindowGeometry = geometry();
	}

	QMainWindow::moveEvent(event);
}

void MainWindow::resizeEvent(QResizeEvent *event)
{
	if(!isFullScreen() && !isMaximized())
	{
		if(currentGameType == 1)
			onePlayerWindowGeometry = geometry();
		else if(currentGameType == 2)
			twoPlayersWindowGeometry = geometry();
		else if(currentGameType == 3)
			twoPlayersWindowGeometry = geometry();
	}

	QMainWindow::resizeEvent(event);
}

void MainWindow::changeEvent(QEvent *event)
{
	switch (event->type()) {
	case QEvent::LanguageChange:
		ui->retranslateUi(this);
		break;
	default:
		break;
	}
}
