/***************************************************************************
 *   Copyright (C) 2008, 2009, 2010 by Miguel Escudero Gestal
 *   miguelescudero@users.sourceforge.net
 *
 *   This file is part of tubularix
 *
 *   tubularix is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   tubularix is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "gameboard.h"
#include "gametypedialog.h"
#include <QtGui/QMainWindow>
#include <QInputDialog>
#include <QList>
#include <QMessageBox>
#include <QSplitter>
//#include <QTcpServer>
//#include <QTcpSocket>


//forward declarations
class QCloseEvent;
class QTimer;


namespace Ui
{
    class MainWindowClass;
}


class MainWindow : public QMainWindow
{
Q_OBJECT
public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

	GameBoard *gameBoard;
	GameBoard *gameBoard2;

private:
	//enum gameState {};
	//enum gameType {SinglePlayer = 1, TwoPlayersNormal = 2, TwoPlayersChallenge = 3, NetworkServer = 4,
		//NetworkClient = 5};
	Ui::MainWindowClass *ui;
	QString playerName;
	QString language;
	int currentGameType;
	int currentGameMode;
	int startLevel;
	QColor colors[9];
	QRect onePlayerWindowGeometry;
	QRect twoPlayersWindowGeometry;

	int keyFirstEventList[2];
	int keyRepeatEventList[2];
	QTimer *keyFirstTimerPlayer1;
	QTimer *keyFirstTimerPlayer2;
	QTimer *keyRepeatTimerPlayer1;
	QTimer *keyRepeatTimerPlayer2;

	int pauseKey;
	int restartKey;
	int left1Key;
	int right1Key;
	int down1Key;
	int drop1Key;
	int rotateCC1Key;
	int rotateC1Key;
	int left2Key;
	int right2Key;
	int down2Key;
	int drop2Key;
	int rotateCC2Key;
	int rotateC2Key;

	int rotationStyle;

	bool movementKeyRepetition;
	bool rotationKeyRepetition;
	int keyRepetitionDelay;
	int keyRepetitionRate;

	bool customMode;
	int rows;
	int columns;

	QTimer *aiMovementTimer1;
	QTimer *aiMovementTimer2;

	//QTcpSocket tcpClient;
	//QTcpServer tcpServer;
	//QTcpSocket *tcpServerConnection;
	//quint32 nextClientBlockSize;
	//quint32 nextServerBlockSize;
	//QString ip;
	//int port;
	//int networkPlayer;

	QList <int> pieces;
	bool samePieces;
	bool sendDestroyedLines;

	QSplitter *splitter;

private slots:
	void about();
	void aboutQt();
	void selectGameType();
	void showGameOptions();
	void setGameType(int value = 1);
	void setGameMode(int value = 1);
	void setStartLevel(int value = 1);
	void readSettings();
	void readGameSettings();
	//void readNetworkClientSettings();
	//void readNetworkServerSettings();
	void writeSettings();
	void selectLanguage();
	void restoreSizeSettings();

	void changeFullScreen();
	void setNormalScreen();
	void showExtraElements();
	void hideExtraElements();

	void startGame();
	void pauseGame();
	void endGame();
	void showHighScores(int winner = -1);
	int checkHighScore();
	QList<QStringList> readHighScores();
	void writeHighScores();

	void updateFirstKeyPressPlayer1();
	void updateFirstKeyPressPlayer2();
	void updateRepeatKeyPressPlayer1();
	void updateRepeatKeyPressPlayer2();
	void setRepeatKeyPlayer1();
	void setRepeatKeyPlayer2();

	void aiMovement1();
	void aiMovement2();
	void setAiPlayer1(bool value = false);
	void setAiPlayer2(bool value = false);

	//void acceptConnection();
	//void processClientPendingStreams();
	//void processServerPendingStreams();
	//void processKeyStream(QString keyName, int type, bool autorep);
	//void sendClientStream(QString keyName, int type, bool autorep); //type-> 1: keyPress, 0: keyRelease
	//void sendServerStream(QString keyName, int type, bool autorep); //type-> 1: keyPress, 0: keyRelease
	//void closeConnection();

	void addPiece();

	void setSendDestroyedLines(bool value = true);

	void setRotationStyle(int style = 0);

protected:
	void closeEvent(QCloseEvent *event);
	void keyPressEvent(QKeyEvent *event);
	void keyReleaseEvent(QKeyEvent *event);
	void moveEvent (QMoveEvent *event);
	void resizeEvent(QResizeEvent *event);
	void changeEvent(QEvent *event);

signals:
	void gameTypeChanged(int value);
	void gameModeChanged(int value);
	void rotationStyleChanged(int value);
	void startLevelChanged(int value);
	void colorsChanged(QColor *col);
	void screenShot(QPixmap pixmap);
};

#endif // MAINWINDOW_H
