/***************************************************************************
 *   Copyright (C) 2008, 2009, 2010 by Miguel Escudero Gestal
 *   miguelescudero@users.sourceforge.net
 *
 *   This file is part of tubularix
 *
 *   tubularix is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   tubularix is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.
 ***************************************************************************/

#include "piececolorselection.h"
#include <QColorDialog>
#include <QMouseEvent>
#include <QPainter>
#include <QSettings>


//public


PieceColorSelection::PieceColorSelection(QWidget *parent)
		: QWidget(parent)
{
	setMouseTracking(true);

	piece = new Piece;
	for(int i = 0; i < 9; i++)
		colors[i] = QColor(0, 0, 0);

	//background
	setPalette(QPalette(colors[8]));
	setAutoFillBackground(true);
}


//public slots


void PieceColorSelection::setColors(QColor *col)
{
	for(int i = 0; i < 9; i++)
		colors[i] = col[i];
	setPalette(QPalette(colors[8]));
	update();
}


//private


void PieceColorSelection::selectColor(QPainter &painter, int num)
{
	QColor color;
	switch(num)
	{
	case 0:
	case 10:
	case 20:
		color = colors[0];
	break;
	case 1:
	case 11:
	case 21:
		color = colors[1];
	break;
	case 2:
	case 12:
	case 22:
		color = colors[2];
	break;
	case 3:
	case 13:
	case 23:
		color = colors[3];
	break;
	case 4:
	case 14:
	case 24:
		color = colors[4];
	break;
	case 5:
	case 15:
	case 25:
		color = colors[5];
	break;
	case 6:
	case 16:
	case 26:
		color = colors[6];
	break;
	case 7:
	case 17:
	case 27:
		color = colors[7];
	break;
	default:
		color = colors[8];
	break;
	}

	if(num >= 10 && num < 20)
		color.setHsv(color.toHsv().hue(), color.toHsv().saturation(), 100, 255);

	if(num >= 20)
		color.setAlpha(40);

	painter.setBrush(color);
}

void PieceColorSelection::changeColor(int n)
{
	QColor color = QColorDialog::getColor(colors[n], this);
	if(color.isValid())
	{
		colors[n] = color;
		if(n == 8)
			setPalette(QPalette(colors[8]));

		update();
		emit colorsChanged(colors);
	}
}


//protected


void PieceColorSelection::paintEvent(QPaintEvent */*event*/)
{
	QPainter painter(this);
	QPen pen;
	pen.setStyle(Qt::SolidLine);
	painter.setPen(pen);

	for(int i = 0; i < 10; i++)
	{
		for(int j = 0; j < 9; j++)
		{
			selectColor(painter, piecesMatrix[i][j]);
			painter.drawRect(30 + j * 30, 130 + i * 30, 30, 30);
		}
	}

	pen.setStyle(Qt::SolidLine);
	pen.setWidth(2);
	pen.setColor(QColor(220, 220, 220));
	painter.setPen(pen);
	QFont textFont;
	textFont.setPointSize(12);
	painter.setFont(textFont);
	painter.drawText(30, 40, tr("Click on a piece to change its color"));
	painter.drawText(30, 70, tr("You can also change background "));
	painter.drawText(30, 100, tr("and scenario colors"));
}

void PieceColorSelection::mousePressEvent(QMouseEvent *event)
{
	if(event->button() == Qt::LeftButton)
	{
		//scenario or pieces
		for(int i = 0; i < 10; i++)
		{
			for(int j = 0; j < 9; j++)
			{
				if(event->x() >= (30 + j * 30) && event->x() < (30 + (j + 1) * 30)
					&& event->y() >= (130 + i * 30) && event->y() < (130 + (i + 1) * 30))
				{
					changeColor(piecesMatrix[i][j]);
					return;
				}
			}
		}

		//background
		changeColor(8);
	}
}
